﻿using ProjectSportCar.Entities;

namespace ProjectSportCar.Drawnings;

/// <summary>
/// Расширение для класса EntityCar
/// </summary>
public static class ExtentionDrawningCar
{
	/// <summary>
	/// Разделитель для записи информации по объекту в файл
	/// </summary>
	private static readonly string _separatorForObject = ":";

	/// <summary>
	/// Создание объекта из строки
	/// </summary>
	/// <param name="info">Строка с данными для создания объекта</param>
	/// <returns>Объект</returns>
	public static DrawningCar? CreateDrawningCar(this string info)
	{
		string[] strs = info.Split(_separatorForObject);
		EntityCar? car = EntitySportCar.CreateEntitySportCar(strs);
		if (car != null)
		{
			return new DrawningSportCar(car);
		}

		car = EntityCar.CreateEntityCar(strs);
		if (car != null)
		{
			return new DrawningCar(car);
		}

		return null;
	}

	/// <summary>
	/// Получение данных для сохранения в файл
	/// </summary>
	/// <param name="drawningCar">Сохраняемый объект</param>
	/// <returns>Строка с данными по объекту</returns>
	public static string GetDataForSave(this DrawningCar drawningCar)
	{
		string[]? array = drawningCar?.EntityCar?.GetStringRepresentation();

		if (array == null)
		{
			return string.Empty;
		}

		return string.Join(_separatorForObject, array);
	}
}