﻿namespace ProjectSportCar.Entities;

/// <summary>
/// Класс-сущность "Автомобиль"
/// </summary>
public class EntityCar
{
	/// <summary>
	/// Скорость
	/// </summary>
	public int Speed { get; private set; }

	/// <summary>
	/// Вес
	/// </summary>
	public double Weight { get; private set; }

	/// <summary>
	/// Основной цвет
	/// </summary>
	public Color BodyColor { get; private set; }

	/// <summary>
	/// Шаг перемещения автомобиля
	/// </summary>
	public double Step => Speed * 100 / Weight;

	/// <summary>
	/// Конструктор сущности
	/// </summary>
	/// <param name="speed">Скорость</param>
	/// <param name="weight">Вес автомобиля</param>
	/// <param name="bodyColor">Основной цвет</param>
	public EntityCar(int speed, double weight, Color bodyColor)
	{
		Speed = speed;
		Weight = weight;
		BodyColor = bodyColor;
	}

	//TODO Прописать метод

	/// <summary>
	/// Получение строк со значениями свойств объекта класса-сущности
	/// </summary>
	/// <returns></returns>
	public virtual string[] GetStringRepresentation()
	{
		return new[] { nameof(EntityCar), Speed.ToString(), Weight.ToString(), BodyColor.Name };
	}

	/// <summary>
	/// Создание объекта из массива строк
	/// </summary>
	/// <param name="strs"></param>
	/// <returns></returns>
	public static EntityCar? CreateEntityCar(string[] strs)
	{
		if (strs.Length != 4 || strs[0] != nameof(EntityCar))
		{
			return null;
		}

		return new EntityCar(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
	}
}