﻿using Microsoft.Extensions.Logging;
using ProjectSportCar.CollectionGenericObjects;
using ProjectSportCar.Drawnings;

namespace ProjectSportCar;

/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>
public partial class FormCarCollection : Form
{
	/// <summary>
	/// Хранилише коллекций
	/// </summary>
	private readonly StorageCollection<DrawningCar> _storageCollection;

	/// <summary>
	/// Компания
	/// </summary>
	private AbstractCompany? _company = null;

	/// <summary>
	/// Логер
	/// </summary>
	private readonly ILogger _logger;

	/// <summary>
	/// Конструктор
	/// </summary>
	public FormCarCollection(ILogger<FormCarCollection> logger)
	{
		InitializeComponent();
		_storageCollection = new();
		_logger = logger;
	}

	/// <summary>
	/// Выбор компании
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
	{
		panelCompanyTools.Enabled = false;
	}

	/// <summary>
	/// Добавление автомобиля
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonAddCar_Click(object sender, EventArgs e)
	{
		FormCarConfig form = new();
		// TODO передать метод
		form.Show();
	}

	/// <summary>
	/// Добавление автомобиля в коллекцию
	/// </summary>
	/// <param name="car"></param>
	private void SetCar(DrawningCar? car)
	{
		if (_company == null || car == null)
		{
			return;
		}

		if (_company + car)
		{
			MessageBox.Show("Объект добавлен");
			pictureBox.Image = _company.Show();
		}
		else
		{
			MessageBox.Show("Не удалось добавить объект");
		}
	}

	/// <summary>
	/// Удаление объекта
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonRemoveCar_Click(object sender, EventArgs e)
	{
		if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
		{
			return;
		}

		if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
		{
			return;
		}

		int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
		if (_company - pos)
		{
			MessageBox.Show("Объект удален");
			pictureBox.Image = _company.Show();
		}
		else
		{
			MessageBox.Show("Не удалось удалить объект");
		}
	}

	/// <summary>
	/// Передача объекта в другую форму
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonGoToCheck_Click(object sender, EventArgs e)
	{
		if (_company == null)
		{
			return;
		}

		DrawningCar? car = null;
		int counter = 100;
		while (car == null)
		{
			car = _company.GetRandomObject();
			counter--;
			if (counter <= 0)
			{
				break;
			}
		}

		if (car == null)
		{
			return;
		}

		FormSportCar form = new()
		{
			SetCar = car
		};
		form.ShowDialog();
	}

	/// <summary>
	/// Перерисовка коллекции
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonRefresh_Click(object sender, EventArgs e)
	{
		if (_company == null)
		{
			return;
		}

		pictureBox.Image = _company.Show();
	}

	/// <summary>
	/// Добавление коллекции
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonCollectionAdd_Click(object sender, EventArgs e)
	{
		if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
		{
			MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			return;
		}

		CollectionType collectionType = CollectionType.None;
		if (radioButtonMassive.Checked)
		{
			collectionType = CollectionType.Massive;
		}
		else if (radioButtonList.Checked)
		{
			collectionType = CollectionType.List;
		}

		_storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
		RerfreshListBoxItems();
	}

	/// <summary>
	/// Удаление коллекции
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonCollectionDel_Click(object sender, EventArgs e)
	{
		// TODO прописать логику удаления элемента из коллекции
		// нужно убедиться, что есть выбранная коллекция
		// спросить у пользователя через MessageBox, что он подтверждает, что хочет удалить запись
		// удалить и обновить ListBox
	}

	/// <summary>
	/// Обновление списка в listBoxCollection
	/// </summary>
	private void RerfreshListBoxItems()
	{
		listBoxCollection.Items.Clear();
		for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
		{
			string? colName = _storageCollection.Keys?[i].Name;
			if (!string.IsNullOrEmpty(colName))
			{
				listBoxCollection.Items.Add(colName);
			}
		}

	}

	/// <summary>
	/// Создание компании
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonCreateCompany_Click(object sender, EventArgs e)
	{
		if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
		{
			MessageBox.Show("Коллекция не выбрана");
			return;
		}

		ICollectionGenericObjects<DrawningCar>? collection = _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
		if (collection == null)
		{
			MessageBox.Show("Коллекция не проинициализирована");
			return;
		}

		switch (comboBoxSelectorCompany.Text)
		{
			case "Хранилище":
				_company = new CarSharingService(pictureBox.Width, pictureBox.Height, collection);
				break;
		}

		panelCompanyTools.Enabled = true;
		RerfreshListBoxItems();
	}

	/// <summary>
	/// Обработка нажатия "Сохранение"
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
	{
		if (saveFileDialog.ShowDialog() == DialogResult.OK)
		{
			try
			{
				_storageCollection.SaveData(saveFileDialog.FileName);
				MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
				_logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
				_logger.LogError("Ошибка: {Message}", ex.Message);
			}
		}
	}

	/// <summary>
	/// Обработка нажатия "Загрузка"
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
	{
		// TODO продумать логику
	}

	/// <summary>
	/// Сортировка по типу
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonSortByType_Click(object sender, EventArgs e)
	{
		CompareCars(new DrawningCarCompareByType());
	}

	/// <summary>
	/// Сортировка по цвету
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonSortByColor_Click(object sender, EventArgs e)
	{
		CompareCars(new DrawningCarCompareByColor());
	}

	/// <summary>
	/// Сортировка по сравнителю
	/// </summary>
	/// <param name="comparer">Сравнитель объектов</param>
	private void CompareCars(IComparer<DrawningCar?> comparer)
	{
		if (_company == null)
		{
			return;
		}

		_company.Sort(comparer);
		pictureBox.Image = _company.Show();
	}
}