﻿using ProjectSportCar.Drawnings;

namespace ProjectSportCar;

/// <summary>
/// Форма конфигурации объекта
/// </summary>
public partial class FormCarConfig : Form
{
	/// <summary>
	/// Объект - прорисовка автомобиля
	/// </summary>
	private DrawningCar? _car;

	/// <summary>
	/// Событие для передачи объекта
	/// </summary>
	private event CarDelegate? CarDelegate;

	/// <summary>
	/// Конструктор
	/// </summary>
	public FormCarConfig()
	{
		panelRed.MouseDown += Panel_MouseDown;
		panelGreen.MouseDown += Panel_MouseDown;
		panelBlue.MouseDown += Panel_MouseDown;
		panelYellow.MouseDown += Panel_MouseDown;
		panelWhite.MouseDown += Panel_MouseDown;
		panelGray.MouseDown += Panel_MouseDown;
		panelBlack.MouseDown += Panel_MouseDown;
		panelPurple.MouseDown += Panel_MouseDown;

		// TODO buttonCancel.Click привязать анонимный метод через lambda с закрытием формы
		InitializeComponent();
	}

	/// <summary>
	/// Привязка внешнего метода к событию
	/// </summary>
	/// <param name="carDelegate"></param>
	public void AddEvent(CarDelegate carDelegate)
	{
		CarDelegate += carDelegate;
	}

	/// <summary>
	/// Прорисовка объекта
	/// </summary>
	private void DrawObject()
	{
		Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
		Graphics gr = Graphics.FromImage(bmp);
		_car?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
		_car?.SetPosition(15, 15);
		_car?.DrawTransport(gr);
		pictureBoxObject.Image = bmp;
	}

	/// <summary>
	/// Передаем информацию при нажатии на Label
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void LabelObject_MouseDown(object sender, MouseEventArgs e)
	{
		(sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
	}

	/// <summary>
	/// Проверка получаемой информации (ее типа на соответствие требуемому)
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void PanelObject_DragEnter(object sender, DragEventArgs e)
	{
		e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
	}

	/// <summary>
	/// Действия при приеме перетаскиваемой информации
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void PanelObject_DragDrop(object sender, DragEventArgs e)
	{
		switch (e.Data?.GetData(DataFormats.Text)?.ToString())
		{
			case "labelSimpleObject":
				_car = new DrawningCar((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
				break;
			case "labelModifiedObject":
				_car = new DrawningSportCar((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White,
					Color.Black, checkBoxBodyKit.Checked, checkBoxWing.Checked, checkBoxSportLine.Checked);
				break;
		}

		DrawObject();
	}

	/// <summary>
	/// Передаем информацию при нажатии на Panel
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void Panel_MouseDown(object? sender, MouseEventArgs e)
	{
		// TODO отправка цвета в Drag&Drop
	}

	// TODO Реализовать логику смены цветов: основного и дополнительного (для продвинутого объекта)

	/// <summary>
	/// Передача объекта
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonAdd_Click(object sender, EventArgs e)
	{
		if (_car != null)
		{
			CarDelegate?.Invoke(_car);
			Close();
		}
	}
}