﻿using ProjectSportCar.Drawnings;
using ProjectSportCar.MovementStrategy;

namespace ProjectSportCar;

/// <summary>
/// Форма работы с объектом "Спортивный автомобиль"
/// </summary>
public partial class FormSportCar : Form
{
	/// <summary>
	/// Поле-объект для прорисовки объекта
	/// </summary>
	private DrawningCar? _drawningCar;

	/// <summary>
	/// Стратегия перемещения
	/// </summary>
	private AbstractStrategy? _strategy;

	/// <summary>
	/// Получение объекта 
	/// </summary>
	public DrawningCar SetCar
	{
		set
		{
			_drawningCar = value;
			_drawningCar.SetPictureSize(pictureBoxSportCar.Width, pictureBoxSportCar.Height);
			comboBoxStrategy.Enabled = true;
			_strategy = null;
			Draw();
		}
	}

	/// <summary>
	/// Конструктор формы
	/// </summary>
	public FormSportCar()
	{
		InitializeComponent();
		_strategy = null;
	}

	/// <summary>
	/// Метод прорисовки машины
	/// </summary>
	private void Draw()
	{
		if (_drawningCar == null)
		{
			return;
		}

		Bitmap bmp = new(pictureBoxSportCar.Width, pictureBoxSportCar.Height);
		Graphics gr = Graphics.FromImage(bmp);
		_drawningCar.DrawTransport(gr);
		pictureBoxSportCar.Image = bmp;
	}

	/// <summary>
	/// Перемещение объекта по форме (нажатие кнопок навигации)
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonMove_Click(object sender, EventArgs e)
	{
		if (_drawningCar == null)
		{
			return;
		}

		string name = ((Button)sender)?.Name ?? string.Empty;
		bool result = false;
		switch (name)
		{
			case "buttonUp":
				result = _drawningCar.MoveTransport(DirectionType.Up);
				break;
			case "buttonDown":
				result = _drawningCar.MoveTransport(DirectionType.Down);
				break;
			case "buttonLeft":
				result = _drawningCar.MoveTransport(DirectionType.Left);
				break;
			case "buttonRight":
				result = _drawningCar.MoveTransport(DirectionType.Right);
				break;
		}

		if (result)
		{
			Draw();
		}
	}

	/// <summary>
	/// Обработка нажатия кнопки "Шаг"
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonStrategyStep_Click(object sender, EventArgs e)
	{
		if (_drawningCar == null)
		{
			return;
		}

		if (comboBoxStrategy.Enabled)
		{
			_strategy = comboBoxStrategy.SelectedIndex switch
			{
				0 => new MoveToCenter(),
				1 => new MoveToBorder(),
				_ => null,
			};
			if (_strategy == null)
			{
				return;
			}
			_strategy.SetData(new MoveableCar(_drawningCar), pictureBoxSportCar.Width, pictureBoxSportCar.Height);
		}

		if (_strategy == null)
		{
			return;
		}

		comboBoxStrategy.Enabled = false;
		_strategy.MakeStep();
		Draw();

		if (_strategy.GetStatus() == StrategyStatus.Finish)
		{
			comboBoxStrategy.Enabled = true;
			_strategy = null;
		}
	}
}